#include "OEMSolver.h"
#include "NonLinUpdater.h"

namespace AMDiS {

  template<typename VectorType>
  void Newton<VectorType>::init()
  {
    TEST_EXIT(this->vectorCreator)("no vectorCreator\n");
    b = this->vectorCreator->create();
  }

  template<typename VectorType>
  void Newton<VectorType>::exit()
  {
    if(b) this->vectorCreator->free(b);
  }

  template<typename VectorType>
  int Newton<VectorType>::nlsolve(MatVecMultiplier<VectorType> *matVec,
				  VectorType *x, VectorType *d,
				  Preconditioner<VectorType> *lPrecon,
				  Preconditioner<VectorType> *rPrecon)
  {
    FUNCNAME("Newton::nlsolve");

    //DOFVector<T> *d = problem->getRHS();
    //DOFVector<T> *x = problem->getSolution();;

    *b = *d;

    //   // copy operators from fh to b
    //   std::vector<Operator*>::iterator op;
    //   std::vector<double*>::iterator fac;
    //   for(op = d->getOperatorsBegin(), 
    // 	fac = d->getOperatorFactorBegin();
    //       op != d->getOperatorsEnd();
    //       ++op, ++fac) 
    //   {
    //     b->addOperator(*op, *fac);
    //   }

    double err = 0.0, err_old = -1.0;
    int    iter, n;

    INFO(this->info,2)("iter. |     this->residual |     red. |    n |\n");

    for (iter = 1; iter <= this->maxIter; iter++)
      {
	/*--- Assemble DF(x) and F(x) ----------------------------------------------*/
	this->nonLinUpdater->update(/*x,*/true, b);
	/*--- Initial guess is zero ------------------------------------------------*/
	d->set(0.0);
	/*--- solve linear system --------------------------------------------------*/
	n = solveLinearSystem(matVec, b, d, lPrecon, rPrecon);
	/*--- x = x - d ------------------------------------------------------------*/
	*x -= *d;

	if(this->usedNorm == NO_NORM || this->usedNorm == L2_NORM)
	  err = L2Norm(d);  // sollte hier nicht b genommen werden (s. NewtonS) ?
	else
	  err = H1Norm(d);  // sollte hier nicht b genommen werden (s. NewtonS) ?
    

	if (iter == 1)  this->initial_residual = err;

	if (err_old <= 0) {
	  INFO(this->info,2)("%5d | %12.5e | -------- | %4d |\n", iter, err, n);
	} else {
	  INFO(this->info,2)("%5d | %12.5e | %8.2e | %4d |\n", 
			     iter, err, err/err_old, n);
	}

	if ((this->residual = err) < this->tolerance)
	  {
	    INFO(this->info,4)("finished successfully\n");
	    return(iter);
	  }
	err_old = err;
      }

    if (this->info < 2)
      INFO(this->info,1)("iter. %d, residual: %12.5e\n", iter, err);
    INFO(this->info,1)("tolerance %e not reached\n", this->tolerance);

    this->residual = err;

    return(iter);
  }

}
