// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file NewtonS.h */

#ifndef AMDIS_newTONS_H
#define AMDIS_newTONS_H

#if 0

#include "CreatorInterface.h"

namespace AMDiS {

  // ============================================================================
  // ===== class NewtonS ========================================================
  // ============================================================================

  /**
   * \ingroup Solver
   * 
   * \brief
   * Implements the newton method with step size control for solving a non linear
   * system. Sub class of NonLinSolver.
   */
  template<typename VectorType>
  class NewtonS : public NonLinSolver<VectorType>
  {
  public:
    /** \brief
     * Creator class used in the NonLinSolverMap.
     */
    class Creator : public NonLinSolverCreator<VectorType> 
    {
    public:
      virtual ~Creator() {};

      /** \brief
       * Returns a new NewtonS object.
       */
      NonLinSolver<VectorType>* create() { 
	return new NewtonS(this->name, this->linearSolver, this->nonLinUpdater); 
      };
    };

    /** \brief
     * Calls constructor of base class NonLinSolver
     */
    NewtonS(std::string name_,
	    OEMSolver *linSolver_,
	    NonLinUpdater<VectorType> *updater)
      : NonLinSolver<VectorType>(name_, linSolver_, updater),
	b(NULL),
	y(NULL),
	restart(0)
    {
      GET_PARAMETER(0, this->name + "->restart", "%d", &restart);  
      TEST_EXIT(restart)("restart not set!\n");
    };

  private:
    /** \brief
     * realisation of NonLinSolver::init
     */
    void init();

    /** \brief
     * realisation of NonLinSolver::nlsolve
     */
    int  nlsolve(MatVecMultiplier<VectorType> *matVec, 
		 VectorType *x, VectorType *rhs,
		 Preconditioner<VectorType> *lPrecon = NULL,
		 Preconditioner<VectorType> *rPrecon = NULL);

    /** \brief
     * realisation of NonLinSolver::exit
     */
    void exit();

  private:
    VectorType *b;       /**< \brief internal used data */
    VectorType *y;       /**< \brief internal used data */
    int restart;     /**< \brief internal used data */
  };

}

#include "NewtonS.hh"

#endif 

#endif // AMDIS_newTONS_H
