// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file ProblemInterpolScal.h */

#ifndef AMDIS_PROBLEMINTERPOLSCAL_H
#define AMDIS_PROBLEMINTERPOLSCAL_H

#include "ProblemScal.h"

namespace AMDiS {

class AdaptInfo;

// ============================================================================
// ===== class ProblemInterpolScal ============================================
// ============================================================================

/** \brief
 * Interpolates a given function adaptive on spaceProblems mesh.
 */
class ProblemInterpolScal : public ProblemScal
{
public:
  /** \brief
   * Constructor. fct will be interpolated on the mesh of spaceProblem.
   * grdFct is used, if H1 error should be used for estimation. It points
   * to the gradient of fct.
   */
  ProblemInterpolScal(const char *name_,
		      ProblemScal *spaceProblem,
		      AbstractFunction<double, WorldVector<double> > *fct,
		      AbstractFunction<WorldVector<double>, WorldVector<double> > *grdFct);

  /** \brief
   * no system assemblage.
   */
  virtual void buildbeforeRefine(AdaptInfo *adaptInfo, Flag) {};

  /** \brief
   * no system assemblage.
   */
  virtual void buildbeforeCoarsen(AdaptInfo *adaptInfo, Flag) {};

  /** \brief
   * no system assemblage.
   */
  virtual void buildAfterCoarsen(AdaptInfo *adaptInfo, Flag) {};

  /** \brief
   * no equation system ins solved. Instead fct is interpolated to uh.
   */
  virtual void solve(AdaptInfo *adaptInfo);

  /** \brief
   * True H1 or L2 error is calculated.
   */
  virtual void estimate(AdaptInfo *adaptInfo, double);

protected:
  /** \brief
   * function to interpolate.
   */
  AbstractFunction<double, WorldVector<double> > *interpolFct_;

  /** \brief
   * gradient of \ref interpolFct_. Used for H1 error in estimate().
   */
  AbstractFunction<WorldVector<double>, WorldVector<double> > *grdInterpolFct_;
};

}

#endif
