#include "ProblemInterpolVec.h"
#include "Mesh.h"
#include "Error.h"
#include "SystemVector.h"
#include "AdaptInfo.h"

namespace AMDiS {

  ProblemInterpolVec::ProblemInterpolVec(const char *nameStr,
					 ProblemVec *spaceProblem,
					 std::vector<AbstractFunction<double, WorldVector<double> >*> *fct,
					 std::vector<AbstractFunction<WorldVector<double>, WorldVector<double> >*> *grdFct)
    : ProblemVec(nameStr),
      interpolFct_(fct),
      grdInterpolFct_(grdFct)
  {
    Flag adoptFlag = 
      INIT_SYSTEM | 
      INIT_MESH |
      INIT_FE_SPACE;

    Flag initFlag = INIT_ALL & ~adoptFlag & ~INIT_SOLVER & ~INIT_ESTIMATOR;
  
    initialize(initFlag, spaceProblem, adoptFlag);
  }

  void ProblemInterpolVec::solve(AdaptInfo *adaptInfo) {
    int size = static_cast<int>(meshes.size());
    for (int i = 0; i < size; i++) {
      meshes[i]->dofCompress();
    }
    solution->interpol(interpolFct_);
  }

  void ProblemInterpolVec::estimate(AdaptInfo *adaptInfo, double) {
    FUNCNAME("ProblemIterpolVec::estimate()");
    double errMax = 0.0, errSum = 0.0;
    int errorNorm = 0;
    int size = static_cast<int>(interpolFct_ ? 
				interpolFct_->size() : 
				grdInterpolFct_->size());

    int relErr = 0;
    GET_PARAMETER(0, name + "->rel error", "%d", &relErr);

    if (grdInterpolFct_) 
      errorNorm = 1;
    else
      if(interpolFct_) errorNorm = 2;

    switch (errorNorm) {
    case 1:
      for (int i = 0; i < size; i++) {
	errSum = Error<double>::H1Err((*(*grdInterpolFct_)[i]), 
				      *(solution->getDOFVector(i)), 
				      relErr, &errMax, true, i);
	adaptInfo->setEstSum(errSum, i);
	adaptInfo->setEstMax(errMax, i);
      }
      break;
    case 2:
      for (int i = 0; i < size; i++) {
	errSum = Error<double>::L2Err((*(*interpolFct_)[i]), 
				      *(solution->getDOFVector(i)), 
				      relErr, &errMax, true, i);
	adaptInfo->setEstSum(errSum, i);
	adaptInfo->setEstMax(errMax, i);
      }
      break;
    default: ERROR_EXIT("invalid error norm\n");
    }

    MSG("estimate: %e\n", errSum);
  }

}
