#include <algorithm>
#include <cstring>

#include "ValueWriter.h"
#include "DOFVector.h"
#include "DOFAdmin.h"
#include "ElInfo.h"
#include "BasisFunction.h"

namespace AMDiS {

  const DOFAdmin* ValueWriter::admin = NULL;
  FILE* ValueWriter::valueFile = NULL;
  DOFVector<double>* ValueWriter::valueVec = NULL;
  DOFVector<int>* ValueWriter::interpPointInd = NULL;
  DOFVector< std::list<WorldVector<double> > >* ValueWriter::dofCoords = NULL;
  int ValueWriter::ni = 0;

  void ValueWriter::writeValues(DataCollector *dc,
				std::string filename,
				double time,
				int level,
				Flag traverseFlag,
				bool (*writeElem)(ElInfo*))
  {
    FUNCNAME("ValueWriter::writeValues()");

    TEST_EXIT(dc)("no data collector\n");

    std::ofstream file;

    file.open(filename.c_str());

    file << "mesh name: " << dc->getMesh()->getName() << std::endl << std::endl;
    file << "time: " << std::scientific << time << std::endl << std::endl;
    file << "number of values: 1" << std::endl << std::endl;
    file << "value description: " << dc->getValues()->getName() << std::endl;
    file << "number of interpolation points: " << dc->getNumberInterpPoints()
	 << std::endl;
    file << "type: scalar" << std::endl;
    file << "interpolation type: lagrange" << std::endl;
    file << "interpolation degree: " << dc->getFeSpace()->getBasisFcts()->getDegree()
	 << std::endl;
    file << "end of description: " << dc->getValues()->getName()
	 << std::endl << std::endl;   

    /* ----- write vertex values -----*/
    DOFVector<int>::Iterator intPointIt(dc->getInterpPointInd(), USED_DOFS);
    DOFVector<double>::Iterator valueIt(dc->getValues(), USED_DOFS);
    DOFVector< std::list<WorldVector<double> > >::Iterator 
      coordIt(dc->getDofCoords(), USED_DOFS);


    file << "vertex values: " << dc->getValues()->getName() << std::endl;

    for (intPointIt.reset(), valueIt.reset(), coordIt.reset();
	 !intPointIt.end(); 
	 ++intPointIt, ++valueIt, ++coordIt) {

	if (*intPointIt == -2) {
	  for (int i = 0; i < (int) coordIt->size(); i++) {
	    file << std::scientific << *valueIt << std::endl;
	  }
	}
      }

    file << std::endl << std::endl;

    /* ----- write interpolation values ----- */
    file << "interpolation values: " << dc->getValues()->getName() << std::endl;

    for (intPointIt.reset(), valueIt.reset();
	 !intPointIt.end(); 
	 ++intPointIt, ++valueIt) {

	if (*intPointIt >= 0) {
	  file << std::scientific << *valueIt << std::endl;
	}
      }

    file << std::endl << std::endl;

    /* ----- write interpolation points for each simplex */
    file << "element interpolation points: " << dc->getValues()->getName() << std::endl;

    std::vector< std::vector<int> >* interpPoints = dc->getInterpPoints();
    std::vector< std::vector<int> >::iterator it1;
    std::vector<int>::iterator it2;

    for (it1 = interpPoints->begin(); it1 != interpPoints->end(); ++it1) {
      for (it2 = it1->begin(); it2 != it1->end(); ++it2) {
	file << (*it2) << " ";
      }
      file << std::endl;
    }
  
    file << std::endl;

    file.close();
  }


}
